<?php
/**
 * @package	Hikashop Matomo Plugin
 * @version	1.0.2
 * @author	HikaShop
 * @copyright	(C) 2010-2024 HIKARI SOFTWARE. All rights reserved.
 * @license	GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class plgSystemMatomo extends JPlugin
{
	private $order = '';
	private $token_auth = '';
	private $site_id = '';
	private $url = '';
	private $js_url = '';
	private $matomoTracker = '';
	function __construct(&$subject, $config) {
		parent::__construct($subject, $config);

		if(!isset($this->params)){
			$plugin = JPluginHelper::getPlugin('system', 'matomo');
			$this->params = new JRegistry(@$plugin->params);
		}
	}

	public function onOrderStatusListingLoad( &$orderstatus_columns, &$rows){
		$orderstatus_columns['matomo'] = array(
			'text' => JText::_('Matomo'),
			'title' => JText::_('Matomo'),
			'description' => JText::_('ORDER_STATUSES_FOR_WHICH_YOU_WANT_ORDERS_TO_BE_SENT_TO_MATOMO'),
			'key' => 'matomo_statuses',
			'default' => 'confirmed,shipped',
			'type' => 'toggle',
			'trigger' => 'plg.system.matomo.matomoStatusUpdate'
		);
	}

	public function matomoStatusUpdate(&$controller, $elementPkey, $value, $extra){
		return $controller->configstatus($elementPkey,$value, $extra);
	}

	public function onAfterOrderCreate(&$order) {
		$this->sendOrderData($order, true);
	}
	public function onAfterOrderUpdate(&$order) {
		$this->sendOrderData($order, true);
	}

	private function sendOrderData($order, $new) {

		$this->init();

		require_once("matomotracker.php");

		$this->matomoTracker = new MatomoTracker((int)$this->site_id, $this->url);

		$this->matomoTracker->setTokenAuth($this->token_auth);

		if($this->sendOrder($order, $new))
			return;
	}

	private function sendOrder($order, $new) {
		if(empty($order->order_status))
			return false;
		$config = hikashop_config();
		$matomo_statuses = explode(',', $config->get('matomo_statuses', 'confirmed,shipped'));
		$isExportable = in_array($order->order_status, $matomo_statuses);
		$wasNotExportable = $new || !in_array($order->old->order_status, $matomo_statuses);
		$needToBeExported = $isExportable && $wasNotExportable;
		if(!$needToBeExported)
			return false;

		$orderClass = hikashop_get('class.order');
		$this->order = $orderClass->loadFullOrder($order->order_id, false, false);

		if(!empty($this->order->order_ip))
			$this->matomoTracker->setIp($this->order->order_ip);

		$taxes = $this->order->order_shipping_tax + $this->order->order_payment_tax - $this->order->order_discount_tax;
		$subtotal = 0;
		foreach($this->order->products as $product) {
			$this->matomoTracker->addEcommerceItem(
				"(".$product->product_id.') '.$product->order_product_code,
				strip_tags($product->order_product_name),
				$this->getProductCategories($product, false),
				round($product->order_product_price, 2),
				$product->order_product_quantity
			);
			$taxes += round($product->order_product_quantity * $product->order_product_tax, 2);

			$subtotal += round($product->order_product_quantity * $product->order_product_price, 2);
		}

		$this->matomoTracker->doTrackEcommerceOrder(
			$this->order->order_number,
			round($this->order->order_full_price, 2),
			round($subtotal, 2),
			round($taxes, 2),
			round($this->order->order_shipping_price, 2),
			round($this->order->order_discount_price, 2)
		);

		return true;
	}

	public function onBeforeCompileHead(){
		$app = JFactory::getApplication();
		if(version_compare(JVERSION,'4.0','>=') && $app->isClient('administrator'))
			return true;
		if(version_compare(JVERSION,'4.0','<') && $app->isAdmin())
			return true;

		if (!defined('DS'))
			define('DS', DIRECTORY_SEPARATOR);
		if(!include_once(rtrim(JPATH_ADMINISTRATOR,DS).DS.'components'.DS.'com_hikashop'.DS.'helpers'.DS.'helper.php')) return true;

		$this->init();

		$js = "
var _paq = window._paq = window._paq || [];
_paq.push(['trackPageView']);
_paq.push(['enableLinkTracking']);
".$this->getCartJS()."
".$this->getProductJS()."
".$this->getCategoryJS()."
(function() {
	var u='//".$this->js_url."';
	_paq.push(['setTrackerUrl', u+'matomo.php']);
	_paq.push(['setSiteId', '".$this->site_id."']);
	var d=document, g=d.createElement('script'), s=d.getElementsByTagName('script')[0];
	g.async=true; g.src=u+'matomo.js'; s.parentNode.insertBefore(g,s);
})();
		";

		$doc = JFactory::getDocument();
		$doc->addScriptDeclaration($js);
	}

	private function init() {
		$this->token_auth = $this->params->get('token_auth','');
		$this->site_id = $this->params->get('site_id','');
		$this->url = rtrim($this->params->get('matomo_url', ''),'/').'/';
		$this->js_url = str_replace(array('http://', 'https://'), array('' , ''), $this->url);
	}

	private function getCategoryJS() {
		$js = '';

		$option = hikaInput::get()->getString('option');
		$ctrl = hikaInput::get()->getString('ctrl');
		$task = hikaInput::get()->getString('task');
		if(in_array($option, array('com_hikashop', '')) && in_array($ctrl, array('product', 'category')) && $task == 'listing') {
			$category = $this->getCategoryInfo();

			if($category) {
				$js .= "
_paq.push(['setEcommerceView',
	false, // (Required) productSKU
	false, // (Optional) productName
	".json_encode($category->category_name).",
	false // (Optional) price
]);
_paq.push(['trackPageView']);
			";
			}
		}

		return $js;
	}

	private function getProductJS() {
		$js = '';

		$option = hikaInput::get()->getString('option');
		$ctrl = hikaInput::get()->getString('ctrl');
		$task = hikaInput::get()->getString('task');
		if(in_array($option, array('com_hikashop', '')) && $ctrl == 'product' && $task != 'listing') {
			$product = $this->getProductInfo();

			if($product) {
				$js .= "
_paq.push(['setEcommerceView',
	'(".$product->product_id.') '.$product->product_code."', // (Required) productSKU
	".json_encode(strip_tags($product->product_name)).", // (Optional) productName
	".$this->getProductCategories($product)."
	".round(@$product->prices[0]->price_value_with_tax, 2)." // (Optional) price
]);
_paq.push(['trackPageView']);
			";
			}
		}

		return $js;
	}
	private function getCategoryInfo() {
		static $category = null;
		if($category !== null)
			return $category;

		$app = JFactory::getApplication();
		$category_id = (int)hikashop_getCID('category_id');

		if(empty($category_id)) {
			$menu = new stdClass();
			$moduleHelper = hikashop_get('helper.module');
			$config = hikashop_config();
			$config->set('no_css_header', 1);
			$moduleHelper->initialize($menu);
			$data = $menu->params->get('data', false);

			if($data === false) {
				$data = new stdClass();
				$data->hk_category = $menu->params->get('hk_category', false);
				if(!empty($data->hk_category))
					$data->hk_category = (object)$data->hk_category;
			}

			if(isset($data->hk_category) && is_object($data->hk_category)){
				$categoryId = (int)$menu->params->get('category', 0);
				if($categoryId > 0) {
					$cat = $categoryClass->get($categoryId);
					if($cat->category_type == 'manufacturer')
						$menu->params->set('content_type', 'manufacturer');
				}
				if(!empty($data->hk_category->category))
					$menu->params->set('selectparentlisting', (int)$data->hk_category->category);
			}
			$category_id = $menu->params->get('selectparentlisting');
		}
		$category = false;
		if(!empty($category_id)) {
			$categoryClass = hikashop_get('class.category');
			$category = $categoryClass->get($category_id);
		}
		return $category;
	}
	private function getProductInfo() {
		static $product = null;
		if($product !== null)
			return $product;

		$app = JFactory::getApplication();
		$product_id = (int)hikashop_getCID('product_id');
		$menus = $app->getMenu();
		$menu = $menus->getActive();
		if(empty($menu) && !empty($Itemid)) {
			$menus->setActive($Itemid);
			$menu = $menus->getItem($Itemid);
		}
		if(empty($product_id) && is_object($menu)) {
			if(!empty($menu)) {
				if(HIKASHOP_J30)
					$menuParams = $menu->getParams();
				else
					$menuParams = @$menu->params;
			}
			$product_id = $menuParams->get('product_id');
		}
		$product = false;
		if(!empty($product_id)) {
			$productClass = hikashop_get('class.product');
			$product = $productClass->get($product_id);
			if(!empty($product) && $product->product_type == 'variant') {
				$product = $productClass->get($product->product_parent_id);
			}
			$ids = array($product->product_id);
			$currencyClass = hikashop_get('class.currency');
			$currencyClass->getProductPrices($product, $ids, array());
		}
		return $product;
	}

	private function getCartJS() {
		$cartClass = hikashop_get('class.cart');
		$cart = $cartClass->getFullCart();

		$js = '';
		if(!empty($cart)) {
			$total = 0;
			if(!empty($cart->total->prices[0]->price_value_with_tax))
				$total = $cart->total->prices[0]->price_value_with_tax;
			if(!empty($cart->products)) {
				$config = hikashop_config();
				$group = $config->get('group_options', 0);
				foreach($cart->products as $k => $product) {
					if(empty($product->cart_product_quantity) || substr($k,0,1) === 'p')
						continue;
					if($group && $product->cart_product_option_parent_id)
						continue;
					$js .= "
_paq.push(['addEcommerceItem',
	'(".$product->product_id.') '.$product->product_code."', // (Required) productSKU
	".json_encode(strip_tags($product->product_name)).", // (Optional) productName
	".$this->getProductCategories($product)."
	".round(@$product->prices[0]->price_value_with_tax, 2).", // (Recommended) price
	".$product->cart_product_quantity." // (Optional, defaults to 1) quantity
]);
					";
				}
				if(!empty($js))
					$js .= "_paq.push(['trackEcommerceCartUpdate', ".round($total,2)."]);";
			}
		}
		return $js;
	}

	private function getProductCategories(&$product, $js = true) {
		if(!isset($product->product_type)) {
			$productClass = hikashop_get('class.product');
			$data = $productClass->get($product->product_id);
			$product->product_type = $data->product_type;
			$product->product_parent_id = $data->product_parent_id;
		}
		$database = JFactory::getDBO();
		$query = 'SELECT b.category_name '.
			' FROM '.hikashop_table('product_category').' AS a '.
			' LEFT JOIN '.hikashop_table('category').' AS b ON a.category_id = b.category_id '.
			' WHERE a.product_id = '.(($product->product_type == 'variant' && !empty($product->product_parent_id)) ? (int)$product->product_parent_id : (int)$product->product_id).
			' AND b.category_published = 1 '.
			' ORDER BY a.product_category_id ASC';
		$database->setQuery($query, 0, 5);
		$categoriesData = $database->loadObjectList();
		if($js) {
			$categoriesText = '';
			if(!empty($categoriesData)) {
				$categories = array();
				foreach($categoriesData as $category) {
					$categories[] = json_encode($category->category_name);
				}
				$categoriesText = "[".implode(', ', $categories)."], // (Optional) productCategory";
			}
			return $categoriesText;
		}
		if(!empty($categoriesData)) {
			$categories = array();
			foreach($categoriesData as $category) {
				$categories[] = $category->category_name;
			}
			return $categories;
		}
		return false;
	}
}
